package xyz.lychee.lagfixer.libs.adventure.text.minimessage;

import java.util.List;
import java.util.Objects;
import java.util.function.Supplier;
import xyz.lychee.lagfixer.libs.adventure.text.minimessage.tag.Tag;
import xyz.lychee.lagfixer.libs.adventure.text.minimessage.tag.resolver.ArgumentQueue;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.annotations.Nullable;

final class ArgumentQueueImpl<T extends Tag.Argument> implements ArgumentQueue {
   private final Context context;
   final List<T> args;
   private int ptr = 0;

   ArgumentQueueImpl(final Context context, final List<T> args) {
      this.context = context;
      this.args = args;
   }

   @NotNull
   public @NotNull T pop() {
      if (!this.hasNext()) {
         throw this.context.newException("Missing argument for this tag!", this);
      } else {
         return (Tag.Argument)this.args.get(this.ptr++);
      }
   }

   @NotNull
   public @NotNull T popOr(final @NotNull String errorMessage) {
      Objects.requireNonNull(errorMessage, "errorMessage");
      if (!this.hasNext()) {
         throw this.context.newException(errorMessage, this);
      } else {
         return (Tag.Argument)this.args.get(this.ptr++);
      }
   }

   @NotNull
   public @NotNull T popOr(final @NotNull Supplier<String> errorMessage) {
      Objects.requireNonNull(errorMessage, "errorMessage");
      if (!this.hasNext()) {
         throw this.context.newException((String)Objects.requireNonNull((String)errorMessage.get(), "errorMessage.get()"), this);
      } else {
         return (Tag.Argument)this.args.get(this.ptr++);
      }
   }

   @Nullable
   public @Nullable T peek() {
      return this.hasNext() ? (Tag.Argument)this.args.get(this.ptr) : null;
   }

   public boolean hasNext() {
      return this.ptr < this.args.size();
   }

   public void reset() {
      this.ptr = 0;
   }

   public String toString() {
      return this.args.toString();
   }
}
